@@ -1,13 +1,14 @@
+from typing import Any, Dict, List, Union
 import numpy as np
 import pandas as pd
 
+from ..helpers.typing_annotations import ShopApi, ShopDatatypes, XyType
 from ..helpers.time import get_shop_datetime, get_shop_timestring
 from ..helpers.timeseries import create_constant_time_series, get_timestamp_indexed_series, resample_resolution
 
-
-def get_attribute_value(shop_api, object_name, object_type, attribute_name, datatype, dataframe=True):
+def get_attribute_value(shop_api:ShopApi, object_name:str, object_type:str, attribute_name:str, datatype:str, dataframe:bool=True) -> ShopDatatypes:
     value = None
-    if datatype == 'int': 
+    if datatype == 'int':
         value = shop_api.GetIntValue(object_type, object_name, attribute_name)
     elif datatype == 'int_array':
         value = list(shop_api.GetIntArray(object_type, object_name, attribute_name))
@@ -21,6 +22,8 @@ def get_attribute_value(shop_api, object_name, object_type, attribute_name, data
             value = None
     elif datatype == 'string':
         value = shop_api.GetStringValue(object_type, object_name, attribute_name)
+    elif datatype == 'string_array':
+        value = shop_api.GetStringArray(object_type, object_name, attribute_name)
     elif datatype == 'xy':
         ref = shop_api.GetXyCurveReference(object_type, object_name, attribute_name)
         x = np.fromiter(shop_api.GetXyCurveX(object_type, object_name, attribute_name), float)
@@ -33,6 +36,17 @@ def get_attribute_value(shop_api, object_name, object_type, attribute_name, data
             else:
                 xy = [[x, y] for x, y in zip(x, y)]
                 value = dict(ref=ref, xy=xy)
+    elif datatype == 'sy':
+        s = shop_api.GetSyCurveS(object_type, object_name, attribute_name)
+        y = np.fromiter(shop_api.GetSyCurveY(object_type, object_name, attribute_name), float)
+        if y.size == 0:
+            value = None
+        else:
+            if dataframe:
+                value = pd.Series(y, index=s)
+            else:
+                sy = [[s, y] for s, y in zip(s, y)]
+                value = dict(sy=sy)
     elif datatype == 'xy_array':
         refs = np.fromiter(shop_api.GetXyCurveArrayReferences(object_type, object_name, attribute_name), float)
         n = np.fromiter(shop_api.GetXyCurveArrayNPoints(object_type, object_name, attribute_name), int)
@@ -56,14 +70,14 @@ def get_attribute_value(shop_api, object_name, object_type, attribute_name, data
                     offset += n_items
     elif datatype == 'xyt':
         tz_name = get_shop_timzone_name(shop_api)
-        start = get_shop_datetime(shop_api.GetStartTime(),tz_name)
-        end = get_shop_datetime(shop_api.GetEndTime(),tz_name)
+        start = get_shop_datetime(shop_api.GetStartTime(), tz_name)
+        end = get_shop_datetime(shop_api.GetEndTime(), tz_name)
         value = get_xyt_attribute(shop_api, object_name, object_type, attribute_name, start, end, dataframe)
     elif datatype == 'txy':
         start_time = shop_api.GetTxySeriesStartTime(object_type, object_name, attribute_name)
         if start_time:
             tz_name = get_shop_timzone_name(shop_api)
-            start_time = get_shop_datetime(start_time,tz_name)
+            start_time = get_shop_datetime(start_time, tz_name)
             t = shop_api.GetTxySeriesT(object_type, object_name, attribute_name)
             y = shop_api.GetTxySeriesY(object_type, object_name, attribute_name)
             time_unit = shop_api.GetTimeUnit()
@@ -73,7 +87,7 @@ def get_attribute_value(shop_api, object_name, object_type, attribute_name, data
     return value
 
 
-def get_xyt_attribute(shop_api, object_name, object_type, attribute_name, start, end, dataframe=True):
+def get_xyt_attribute(shop_api:ShopApi, object_name:str, object_type:str, attribute_name:str, start:pd.Timestamp, end:pd.Timestamp, dataframe:bool=True) -> List[XyType]:
     # Get time delta from time unit
     unit = shop_api.GetTimeUnit()
     delta = pd.Timedelta(minutes=1)
@@ -87,8 +101,8 @@ def get_xyt_attribute(shop_api, object_name, object_type, attribute_name, start,
 
     # Identify the indices that should be extracted from the xyt series
     tz_name = get_shop_timzone_name(shop_api)
-    shop_start_time = get_shop_datetime(shop_api.GetStartTime(),tz_name)
-    shop_end_time = get_shop_datetime(shop_api.GetEndTime(),tz_name)
+    shop_start_time = get_shop_datetime(shop_api.GetStartTime(), tz_name)
+    shop_end_time = get_shop_datetime(shop_api.GetEndTime(), tz_name)
     min_time_index = int((start - shop_start_time)/(resolution*delta))
     max_time_index = int((end - shop_start_time)/(resolution*delta))
 
@@ -133,23 +147,24 @@ def get_xyt_attribute(shop_api, object_name, object_type, attribute_name, start,
     return value
 
 
-def get_attribute_info(shop_api, object_type, attribute_name, key=''):
+def get_attribute_info(shop_api:ShopApi, object_type:str, attribute_name:str, key:str='') -> Union[str,Dict[str,str]]:
     if key:
         return shop_api.GetAttributeInfo(object_type, attribute_name, key)
     else:
-        return {key: shop_api.GetAttributeInfo(object_type, attribute_name, key) for key in shop_api.GetValidAttributeInfoKeys()}
+        return {
+            key: shop_api.GetAttributeInfo(object_type, attribute_name, key) for key in shop_api.GetValidAttributeInfoKeys()
+        }
 
 
-def get_object_info(shop_api, object_type, key=''):
+def get_object_info(shop_api:ShopApi, object_type:str, key:str='') -> Union[str,Dict[str,str]]:
     if key:
         return shop_api.GetObjectInfo(object_type, key)
     else:
         return {key: shop_api.GetObjectInfo(object_type, key) for key in shop_api.GetValidObjectInfoKeys()}
 
-
-def set_attribute(shop_api, object_name, object_type, attribute_name, datatype, value):
-    ##Set a attribute in the SHOP core.
-    #datatype = get_attribute_info(shop_api, object_type, attribute_name, 'datatype')
+def set_attribute(shop_api:ShopApi, object_name:str, object_type:str, attribute_name:str, datatype:str, value:ShopDatatypes) -> None:
+    # Set a attribute in the SHOP core.
+    # datatype = get_attribute_info(shop_api, object_type, attribute_name, 'datatype')
     if datatype == 'int':
         shop_api.SetIntValue(object_type, object_name, attribute_name, int(value))
     elif datatype == 'int_array':
@@ -160,6 +175,8 @@ def set_attribute(shop_api, object_name, object_type, attribute_name, datatype,
         shop_api.SetDoubleArray(object_type, object_name, attribute_name, value)
     elif datatype == 'string':
         shop_api.SetStringValue(object_type, object_name, attribute_name, value)
+    elif datatype == 'string_array':
+        shop_api.SetStringArray(object_type, object_name, attribute_name, value)
     elif datatype == 'xy':
         if isinstance(value, pd.Series):
             ref = value.name
@@ -172,6 +189,14 @@ def set_attribute(shop_api, object_name, object_type, attribute_name, datatype,
             x = [x[0] for x in value['xy']]
             y = [x[1] for x in value['xy']]
             shop_api.SetXyCurve(object_type, object_name, attribute_name, value['ref'], x, y)
+    elif datatype == 'sy':
+        if isinstance(value, pd.Series):
+            shop_api.SetSyCurve(object_type, object_name, attribute_name, value.index.to_list(),
+                                value.values)
+        else:
+            s = [s[0] for s in value['sy']]
+            y = [x[1] for x in value['sy']]
+            shop_api.SetSyCurve(object_type, object_name, attribute_name, list(s), y)
     elif datatype == 'xy_array':
         if len(value) == 0:
             return
@@ -184,7 +209,7 @@ def set_attribute(shop_api, object_name, object_type, attribute_name, datatype,
                 ref = np.append(ref, float(df.columns[0]))
                 n = np.append(n, df.size)
                 x = np.append(x, df.index.values)
-                y = np.append(y, df.iloc[:,0].values)
+                y = np.append(y, df.iloc[:, 0].values)
         elif isinstance(value[0], pd.Series):
             for ser in value:
                 ref = np.append(ref, float(ser.name))
@@ -210,8 +235,10 @@ def set_attribute(shop_api, object_name, object_type, attribute_name, datatype,
             df = value
 
         if df.shape[0] == 0:
-            shop_api.SetTxySeries(object_type, object_name, attribute_name,
-                                  get_shop_timestring(time['starttime']), [], [])
+            shop_api.SetTxySeries(
+                object_type, object_name, attribute_name,
+                get_shop_timestring(time['starttime']), [], []
+            )
             return
 
         # Extract data in time interval
@@ -238,7 +265,7 @@ def set_attribute(shop_api, object_name, object_type, attribute_name, datatype,
                     new_row = pd.DataFrame(df[-1:].values, index=[time['endtime']], columns=df.columns)
                 else:
                     new_row = pd.Series(df[-1], index=[time['endtime']])
-                df = df.append(new_row)
+                df = pd.concat([df, new_row])
             df = df.asfreq(freq=freq, method='ffill')
             df = df[:-1]
             time_resolution = pd.Series(data=shop_api.GetTimeResolutionY(), index=shop_api.GetTimeResolutionT())
@@ -251,20 +278,19 @@ def set_attribute(shop_api, object_name, object_type, attribute_name, datatype,
                               t.astype(int), y)
 
 
-def get_time_resolution(shop_api):
+def get_time_resolution(shop_api:ShopApi) -> Dict[str,Any]:
     tz_name = get_shop_timzone_name(shop_api)
-    starttime = get_shop_datetime(shop_api.GetStartTime(),tz_name)
-    endtime = get_shop_datetime(shop_api.GetEndTime(),tz_name)
+    starttime = get_shop_datetime(shop_api.GetStartTime(), tz_name)
+    endtime = get_shop_datetime(shop_api.GetEndTime(), tz_name)
     timeunit = shop_api.GetTimeUnit()
     t = shop_api.GetTimeResolutionT()
     y = shop_api.GetTimeResolutionY()
     timeresolution = get_timestamp_indexed_series(starttime, timeunit, t, y)
     return dict(starttime=starttime, endtime=endtime, timeunit=timeunit, timeresolution=timeresolution)
 
-def get_shop_timzone_name(shop_api):
+def get_shop_timzone_name(shop_api:ShopApi) -> str:
     try:
         tz_name = shop_api.GetTimeZone()
-    except AttributeError:  #For backwards compatability to SHOP 13
+    except AttributeError:  # For backwards compatability to SHOP 13
         tz_name = ""
-    
-    return tz_name
\ No newline at end of file
+    return tz_name
